@@ -1,4 +1,3 @@
-/* $Id: xfburn-burn-data-cd-composition-dialog.h 7506 2009-06-09 01:47:30Z squisher $ */
 /*
  *  Copyright (c) 2005-2007 Jean-François Wauthy (pollux@xfce.org)
  *
@@ -49,9 +48,9 @@ typedef struct
   XfburnBurnDataCompositionBaseDialogClass parent_class;
 } XfburnBurnDataCdCompositionDialogClass;
 
-GType xfburn_burn_data_cd_composition_dialog_get_type ();
+GType xfburn_burn_data_cd_composition_dialog_get_type (void);
 
-GtkWidget *xfburn_burn_data_cd_composition_dialog_new (IsoImage *image);
+GtkWidget *xfburn_burn_data_cd_composition_dialog_new (IsoImage *image, gboolean has_default_name);
 
 G_END_DECLS
 #endif /* XFBURN_BURN_DATA_CD_COMPOSITION_DIALOG_H */
