@@ -1,4 +1,3 @@
-/* $Id: xfburn-data-composition.c 7732 2009-07-16 14:41:23Z squisher $ */
 /*
  * Copyright (c) 2005-2006 Jean-François Wauthy (pollux@xfce.org)
  *
@@ -80,8 +79,7 @@ typedef enum
 
 /* thread parameters */
 typedef struct {
-  char **filenames;
-  int filec;
+  GSList * filelist;
   XfburnDataComposition *dc;
 } ThreadAddFilesCLIParams;
 
@@ -113,6 +111,9 @@ static void save_to_file (XfburnComposition *composition);
 
 static gint directory_tree_sortfunc (GtkTreeModel * model, GtkTreeIter * a, GtkTreeIter * b, gpointer user_data);
 
+static void set_default_name (XfburnDataComposition * dc);
+static gboolean has_default_name (XfburnDataComposition * dc);
+
 static void action_create_directory (GtkAction *, XfburnDataComposition *);
 static void action_clear (GtkAction *, XfburnDataComposition *);
 static void action_remove_selection (GtkAction *, XfburnDataComposition *);
@@ -128,7 +129,7 @@ static void cb_cell_file_edited (GtkCellRenderer * renderer, gchar * path, gchar
 static void cb_content_drag_data_rcv (GtkWidget * widget, GdkDragContext * dc, guint x, guint y, GtkSelectionData * sd,
                                       guint info, guint t, XfburnDataComposition * composition);
 static void cb_content_drag_data_get (GtkWidget * widget, GdkDragContext * dc, GtkSelectionData * data, guint info,
-                                      guint time, XfburnDataComposition * content);
+                                      guint timestamp, XfburnDataComposition * content);
 static void cb_adding_done (XfburnAddingProgress *progress, XfburnDataComposition *dc);
 
 /* thread entry points */
@@ -143,6 +144,7 @@ static gboolean thread_add_file_to_list_with_name (const gchar *name, XfburnData
 static gboolean thread_add_file_to_list (XfburnDataComposition * dc, GtkTreeModel * model, const gchar * path, 
                                          GtkTreeIter * iter, GtkTreeIter * insertion, GtkTreeViewDropPosition position);
 static IsoImage * generate_iso_image (XfburnDataComposition * dc);
+static gboolean show_add_home_question_dialog (void);
                                   
 typedef struct
 {
@@ -172,6 +174,8 @@ typedef struct
   GtkWidget *progress;
   GtkTreeStore *model;
 
+  gchar *default_vol_name;
+
 } XfburnDataCompositionPrivate;
 
 /* globals */
@@ -290,13 +294,6 @@ xfburn_data_composition_init (XfburnDataComposition * composition)
                                 { "text/plain;charset=utf-8", 0, DATA_COMPOSITION_DND_TARGET_TEXT_PLAIN },
                               };
 
-  gchar *vol_name;
-  char timestr[80];
-  struct tm *today;
-  time_t tm;
-  /* FIXME: put i into the class? */
-  static int i = 0;
-
   priv->full_paths_to_add = NULL;
 
   instances++;
@@ -357,17 +354,8 @@ xfburn_data_composition_init (XfburnDataComposition * composition)
   
   priv->entry_volume_name = gtk_entry_new ();
 
-  tm = time (NULL);
-  today = localtime (&tm);
+  set_default_name (composition);
 
-  if (tm && strftime (timestr, 80, "%Y-%m-%d", today))
-    /* Note to translators: first %s is the date in "i18n" format (year-month-day), %d is a running number of compositions */
-    vol_name = g_strdup_printf (_("Data %s~%d"), timestr, ++i);
-  else
-    vol_name = g_strdup_printf ("%s %d", _(DATA_COMPOSITION_DEFAULT_NAME), ++i);
-
-  gtk_entry_set_text (GTK_ENTRY (priv->entry_volume_name), vol_name);
-  g_free (vol_name);
   gtk_box_pack_start (GTK_BOX (hbox), priv->entry_volume_name, FALSE, FALSE, 0);
   gtk_widget_show (priv->entry_volume_name);
   
@@ -512,7 +500,7 @@ cb_begin_burn (XfburnDataDiscUsage * du, XfburnDataComposition * dc)
   
   switch (xfburn_disc_usage_get_disc_type (XFBURN_DISC_USAGE (du))) {
   case CD_DISC:
-    dialog = xfburn_burn_data_cd_composition_dialog_new (image);
+    dialog = xfburn_burn_data_cd_composition_dialog_new (image, has_default_name (dc));
     break;
   case DVD_DISC:
     dialog = xfburn_burn_data_dvd_composition_dialog_new (image);
@@ -940,7 +928,7 @@ action_add_selected_files (GtkAction *action, XfburnDataComposition *dc)
     params->model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->content));
     
     if (selected_paths) {
-      priv->path_where_insert = (GtkTreePath *) (selected_paths->data);
+      priv->path_where_insert = gtk_tree_path_copy ((GtkTreePath *) (selected_paths->data));
 
       gtk_tree_model_get_iter (params->model, &params->iter_where_insert, priv->path_where_insert);
       gtk_tree_model_get (params->model, &params->iter_where_insert, DATA_COMPOSITION_COLUMN_TYPE, &params->type, -1);
@@ -956,6 +944,44 @@ action_add_selected_files (GtkAction *action, XfburnDataComposition *dc)
   }
 }
 
+static void
+set_default_name (XfburnDataComposition * dc)
+{
+  XfburnDataCompositionPrivate *priv = XFBURN_DATA_COMPOSITION_GET_PRIVATE (dc);
+
+  char timestr[80];
+  struct tm *today;
+  time_t tm;
+  /* FIXME: put i into the class? */
+  static int i = 0;
+
+  tm = time (NULL);
+  today = localtime (&tm);
+
+  if (priv->default_vol_name)
+    g_free (priv->default_vol_name);
+
+  if (tm && strftime (timestr, 80, "%Y-%m-%d", today))
+    /* Note to translators: first %s is the date in "i18n" format (year-month-day), %d is a running number of compositions */
+    priv->default_vol_name = g_strdup_printf (_("Data %s~%d"), timestr, ++i);
+  else
+    priv->default_vol_name = g_strdup_printf ("%s %d", _(DATA_COMPOSITION_DEFAULT_NAME), ++i);
+
+  gtk_entry_set_text (GTK_ENTRY (priv->entry_volume_name), priv->default_vol_name);
+}
+
+static gboolean
+has_default_name (XfburnDataComposition * dc)
+{
+  XfburnDataCompositionPrivate *priv = XFBURN_DATA_COMPOSITION_GET_PRIVATE (dc);
+
+  const gchar *name;
+
+  name = gtk_entry_get_text (GTK_ENTRY (priv->entry_volume_name));
+
+  return strcmp (name, priv->default_vol_name) == 0;
+}
+
 static void
 action_clear (GtkAction * action, XfburnDataComposition * dc)
 {
@@ -966,14 +992,14 @@ action_clear (GtkAction * action, XfburnDataComposition * dc)
   model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->content));
   gtk_tree_store_clear (GTK_TREE_STORE (model));
   
-  gtk_entry_set_text (GTK_ENTRY (priv->entry_volume_name), _(DATA_COMPOSITION_DEFAULT_NAME));
+  set_default_name (dc);
 
   xfburn_disc_usage_set_size (XFBURN_DISC_USAGE (priv->disc_usage), 0);
 }
 
 static void
 cb_content_drag_data_get (GtkWidget * widget, GdkDragContext * dc,
-                          GtkSelectionData * data, guint info, guint time, XfburnDataComposition * content)
+                          GtkSelectionData * data, guint info, guint timestamp, XfburnDataComposition * content)
 {
   if (info == DATA_COMPOSITION_DND_TARGET_INSIDE) {
     GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (widget));
@@ -1240,24 +1266,26 @@ thread_add_files_cli (ThreadAddFilesCLIParams *params)
   GtkTreeIter iter;
 
   GtkTreeModel *model;
-  int i;
-  gchar *full_path = NULL;
+  GSList * list_iter;
 
   gdk_threads_enter ();
   model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->content));
   gdk_threads_leave ();
 
-  for (i=0; i<params->filec; i++) {
-    full_path = g_build_filename (params->filenames[i], NULL);
+  for (list_iter = params->filelist; list_iter != NULL; list_iter = g_slist_next (list_iter)) {
+    gchar * full_path = (gchar *) list_iter->data;
+
     g_message ("Adding %s to the data composition... (might take a while)", full_path);
     thread_add_file_to_list (params->dc, model, full_path, &iter, NULL, GTK_TREE_VIEW_DROP_AFTER);  
+
     g_free (full_path);
   }
+  g_slist_free (params->filelist);
   xfburn_adding_progress_done (XFBURN_ADDING_PROGRESS (priv->progress));
 }
 
 static gboolean
-show_add_home_question_dialog ()
+show_add_home_question_dialog (void)
 {
   gboolean ok = TRUE;
 
@@ -1283,74 +1311,79 @@ thread_add_files_action (ThreadAddFilesActionParams *params)
   GtkTreeModel *model = params->model;
   GtkTreeIter iter_where_insert = params->iter_where_insert;
   GtkTreePath *path_where_insert = priv->path_where_insert;
-  const gchar * file = NULL;
+  gchar ** files = NULL;
+  int i;
 
 
-  file = strtok (priv->selected_files, "\n");
-  while (file) {
-    GtkTreeIter iter;
-    gchar *full_path = NULL;
-    
-    if (g_str_has_prefix (file, "file://"))
-      full_path = g_build_filename (&file[7], NULL);
-    else if (g_str_has_prefix (file, "file:"))
-      full_path = g_build_filename (&file[5], NULL);
-    else
-      full_path = g_build_filename (file, NULL);
+  files = g_strsplit (priv->selected_files, "\n", -1);
 
-    if (full_path[strlen (full_path) - 1] == '\r')
-      full_path[strlen (full_path) - 1] = '\0';
+  if (files)
+    for (i=0; files[i] != NULL && files[i][0] != '\0'; i++) {
+      GtkTreeIter iter;
+      gchar *full_path = NULL;
+      
+      if (g_str_has_prefix (files[i], "file://"))
+        full_path = g_build_filename (&files[i][7], NULL);
+      else if (g_str_has_prefix (files[i], "file:"))
+        full_path = g_build_filename (&files[i][5], NULL);
+      else
+        full_path = g_build_filename (files[i], NULL);
 
-    if (strcmp (full_path, g_getenv ("HOME")) == 0) {
-      if (!show_add_home_question_dialog ()) {
-        g_free (full_path);
-        break;
+      if (full_path[strlen (full_path) - 1] == '\r')
+        full_path[strlen (full_path) - 1] = '\0';
+
+      if (strcmp (full_path, g_getenv ("HOME")) == 0) {
+        if (!show_add_home_question_dialog ()) {
+          g_free (full_path);
+          break;
+        }
       }
-    }
 
-    /* add files to the disc content */
-    if (params->type == DATA_COMPOSITION_TYPE_DIRECTORY) {
-      guint64 old_size, size;
-      gchar *humansize = NULL;
-      
-      thread_add_file_to_list (dc, model, full_path, &iter, &iter_where_insert, GTK_TREE_VIEW_DROP_INTO_OR_AFTER);
-      gdk_threads_enter ();
-      gtk_tree_view_expand_row (GTK_TREE_VIEW (priv->content), path_where_insert, FALSE);
-      
-      /* update parent directory size */
-      gtk_tree_model_get (model, &iter_where_insert, DATA_COMPOSITION_COLUMN_SIZE, &old_size, -1);
-      gtk_tree_model_get (model, &iter, DATA_COMPOSITION_COLUMN_SIZE, &size, -1);
-      gdk_threads_leave ();
-      
-      humansize = xfburn_humanreadable_filesize (old_size + size);
-      
-      gdk_threads_enter ();
-      gtk_tree_store_set (GTK_TREE_STORE (model), &iter_where_insert, 
-                          DATA_COMPOSITION_COLUMN_HUMANSIZE, humansize,
-                          DATA_COMPOSITION_COLUMN_SIZE, old_size + size, -1);
-      gdk_threads_leave ();
-      
-      g_free (humansize);
-    } else if (params->type == DATA_COMPOSITION_TYPE_FILE) {
-      GtkTreeIter parent;
-      gboolean has_parent;
+      /* add files to the disc content */
+      if (params->type == DATA_COMPOSITION_TYPE_DIRECTORY) {
+        guint64 old_size, size;
+        gchar *humansize = NULL;
+        
+        thread_add_file_to_list (dc, model, full_path, &iter, &iter_where_insert, GTK_TREE_VIEW_DROP_INTO_OR_AFTER);
+        gdk_threads_enter ();
+        gtk_tree_view_expand_row (GTK_TREE_VIEW (priv->content), path_where_insert, FALSE);
+        
+        /* update parent directory size */
+        gtk_tree_model_get (model, &iter_where_insert, DATA_COMPOSITION_COLUMN_SIZE, &old_size, -1);
+        gtk_tree_model_get (model, &iter, DATA_COMPOSITION_COLUMN_SIZE, &size, -1);
+        gdk_threads_leave ();
+        
+        humansize = xfburn_humanreadable_filesize (old_size + size);
+        
+        gdk_threads_enter ();
+        gtk_tree_store_set (GTK_TREE_STORE (model), &iter_where_insert, 
+                            DATA_COMPOSITION_COLUMN_HUMANSIZE, humansize,
+                            DATA_COMPOSITION_COLUMN_SIZE, old_size + size, -1);
+        gdk_threads_leave ();
+        
+        g_free (humansize);
+      } else if (params->type == DATA_COMPOSITION_TYPE_FILE) {
+        GtkTreeIter parent;
+        gboolean has_parent;
 
-      gdk_threads_enter ();
-      has_parent = gtk_tree_model_iter_parent (model, &parent, &iter_where_insert);
-      gdk_threads_leave ();
-      
-      if (has_parent)
-        thread_add_file_to_list (dc, model, full_path, &iter, &parent, GTK_TREE_VIEW_DROP_INTO_OR_AFTER);  
-      else 
+        gdk_threads_enter ();
+        has_parent = gtk_tree_model_iter_parent (model, &parent, &iter_where_insert);
+        gdk_threads_leave ();
+        
+        if (has_parent)
+          thread_add_file_to_list (dc, model, full_path, &iter, &parent, GTK_TREE_VIEW_DROP_INTO_OR_AFTER);  
+        else 
+          thread_add_file_to_list (dc, model, full_path, &iter, NULL, GTK_TREE_VIEW_DROP_AFTER);  
+      } else {
         thread_add_file_to_list (dc, model, full_path, &iter, NULL, GTK_TREE_VIEW_DROP_AFTER);  
-    } else {
-      thread_add_file_to_list (dc, model, full_path, &iter, NULL, GTK_TREE_VIEW_DROP_AFTER);  
+      }
+      
+      g_free (full_path);
     }
-    
-    g_free (full_path);
+  /* end if files */
+
+  g_strfreev (files);
 
-    file = strtok (NULL, "\n");
-  }
   xfburn_adding_progress_done (XFBURN_ADDING_PROGRESS (priv->progress));
 }
 
@@ -1625,70 +1658,75 @@ cb_content_drag_data_rcv (GtkWidget * widget, GdkDragContext * dc, guint x, guin
   /* drag from the file selector, or nautilus */
   else if (sd->target == gdk_atom_intern ("text/plain;charset=utf-8", FALSE)) {
     ThreadAddFilesDragParams *params;
-    const gchar *file = NULL;
+    gchar **files = NULL;
     gchar *full_paths;
+    int i;
 
     xfburn_adding_progress_show (XFBURN_ADDING_PROGRESS (priv->progress));
 
     full_paths = (gchar *) gtk_selection_data_get_text (sd);
     
-    file = strtok ((gchar *) full_paths, "\n");
-    while (file) {
-      gchar *full_path;
+    files = g_strsplit ((gchar *) full_paths, "\n", -1);
+
+    if (files) {
+
+      for (i=0; files[i] != NULL && files[i][0] != '\0'; i++) {
+        gchar *full_path;
 
 #ifdef HAVE_THUNAR_VFS
-      ThunarVfsPath *vfs_path;
-      GError *vfs_error = NULL;
+        ThunarVfsPath *vfs_path;
+        GError *vfs_error = NULL;
 
-      vfs_path = thunar_vfs_path_new (file, &vfs_error);
+        vfs_path = thunar_vfs_path_new (files[i], &vfs_error);
 
-      if (vfs_error) {
-        g_warning ("Failed to create vfs path for '%s': %s", file, vfs_error->message);
-        g_error_free (vfs_error);
-        continue;
-      }
+        if (vfs_error) {
+          g_warning ("Failed to create vfs path for '%s': %s", files[i], vfs_error->message);
+          g_error_free (vfs_error);
+          continue;
+        }
 
-      if (thunar_vfs_path_get_scheme (vfs_path) != THUNAR_VFS_PATH_SCHEME_FILE)
-        continue;
-      full_path = thunar_vfs_path_dup_string (vfs_path);
+        if (thunar_vfs_path_get_scheme (vfs_path) != THUNAR_VFS_PATH_SCHEME_FILE)
+          continue;
+        full_path = thunar_vfs_path_dup_string (vfs_path);
 
-      thunar_vfs_path_unref (vfs_path);
+        thunar_vfs_path_unref (vfs_path);
 
 #else /* no thunar-vfs */
 
-      if (g_str_has_prefix (file, "file://"))
-        full_path = g_build_filename (&file[7], NULL);
-      else if (g_str_has_prefix (file, "file:"))
-        full_path = g_build_filename (&file[5], NULL);
-      else
-        full_path = g_build_filename (file, NULL);
+        if (g_str_has_prefix (files[i], "file://"))
+          full_path = g_build_filename (&files[i][7], NULL);
+        else if (g_str_has_prefix (files[i], "file:"))
+          full_path = g_build_filename (&files[i][5], NULL);
+        else
+          full_path = g_build_filename (files[i], NULL);
 
-      if (full_path[strlen (full_path) - 1] == '\r')
-        full_path[strlen (full_path) - 1] = '\0';
+        if (full_path[strlen (full_path) - 1] == '\r')
+          full_path[strlen (full_path) - 1] = '\0';
 #endif
 
-      DBG ("Adding path '%s'", full_path);
+        DBG ("Adding path '%s'", full_path);
 
-      /* remember path to add it later in another thread */
-      priv->full_paths_to_add = g_list_append (priv->full_paths_to_add, full_path);
+        /* remember path to add it later in another thread */
+        priv->full_paths_to_add = g_list_append (priv->full_paths_to_add, full_path);
+      }
+      g_strfreev (files);
+      g_free (full_paths);
 
-      file = strtok (NULL, "\n");
+      priv->full_paths_to_add = g_list_reverse (priv->full_paths_to_add);
+      /* FIXME: path_where_insert is always NULL here */
+      priv->path_where_insert = path_where_insert;
+
+      params = g_new (ThreadAddFilesDragParams, 1);
+      params->composition = composition;
+      params->position = position;
+      params->widget = widget;
+
+      /* append a dummy row so that gtk doesn't freak out */
+      gtk_tree_store_append (GTK_TREE_STORE (model), &params->iter_dummy, NULL);
+
+      priv->thread_params = params;
+      g_thread_create ((GThreadFunc) thread_add_files_drag, params, FALSE, NULL);
     }
-    g_free (full_paths);
-
-    priv->full_paths_to_add = g_list_reverse (priv->full_paths_to_add);
-    priv->path_where_insert = path_where_insert;
-
-    params = g_new (ThreadAddFilesDragParams, 1);
-    params->composition = composition;
-    params->position = position;
-    params->widget = widget;
-
-    /* append a dummy row so that gtk doesn't freak out */
-    gtk_tree_store_append (GTK_TREE_STORE (model), &params->iter_dummy, NULL);
-
-    priv->thread_params = params;
-    g_thread_create ((GThreadFunc) thread_add_files_drag, params, FALSE, NULL);
 
     gtk_drag_finish (dc, TRUE, FALSE, t);
   } 
@@ -1715,6 +1753,7 @@ cb_content_drag_data_rcv (GtkWidget * widget, GdkDragContext * dc, guint x, guin
       thunar_vfs_path_list_free (vfs_paths);
 
       priv->full_paths_to_add = g_list_reverse (priv->full_paths_to_add);
+      /* FIXME: path_where_insert is always NULL here */
       priv->path_where_insert = path_where_insert;
 
       params = g_new (ThreadAddFilesDragParams, 1);
@@ -1829,10 +1868,10 @@ fill_image_with_composition (GtkTreeModel *model, IsoImage *image, IsoDir * pare
           g_error ("Failed adding %s as a node to the image: null pointer!", src);
         else if (r == ISO_NODE_NAME_NOT_UNIQUE)
           g_error ("Failed adding %s as a node to the image: node name not unique!", src);
-        /* else if (r == ISO_MEM_ERROR)
-          g_error ("Failed adding %s as a node to the image: memory error!", src); */
+        else if (r == ISO_OUT_OF_MEM)
+          g_error ("Failed adding %s as a node to the image: out of memory!", src);
         else
-          g_error ("Failed adding %s as a node to the image: code %d!", src, r);
+          g_error ("Failed adding %s as a node to the image: code %X!", src, r);
       }
 
       basename = g_path_get_basename (src);
@@ -2152,16 +2191,15 @@ xfburn_data_composition_new (void)
 }
 
 void 
-xfburn_data_composition_add_files (XfburnDataComposition *dc, int filec, char **filenames)
+xfburn_data_composition_add_files (XfburnDataComposition *dc, GSList * filelist)
 {
   XfburnDataCompositionPrivate *priv = XFBURN_DATA_COMPOSITION_GET_PRIVATE (dc);
   ThreadAddFilesCLIParams *params;
   
-  if (filec > 0) {
+  if (filelist != NULL) {
     params = g_new (ThreadAddFilesCLIParams, 1);
 
-    params->filenames = filenames;
-    params->filec = filec;
+    params->filelist = filelist;
     params->dc = dc;
 
     xfburn_adding_progress_show (XFBURN_ADDING_PROGRESS (priv->progress));
