@@ -1,4 +1,3 @@
-/* $Id: xfburn-progress-dialog.h 6049 2008-11-10 06:29:00Z squisher $ */
 /*
  *  Copyright (c) 2005-2006 Jean-François Wauthy (pollux@xfce.org)
  *
@@ -62,7 +61,7 @@ typedef struct
 } XfburnProgressDialogClass;
 
 
-GtkType xfburn_progress_dialog_get_type ();
+GType xfburn_progress_dialog_get_type (void);
 
 void xfburn_progress_dialog_show_buffers (XfburnProgressDialog * dialog, gboolean show);
 void xfburn_progress_dialog_pulse_progress_bar (XfburnProgressDialog * dialog);
